<?php

/*
 
 File: mailbox.php3
 $Author: mhardy $
 $Revision: 1.169.4.18 $
 $Date: 1999/07/30 01:26:52 $
 
 IMP: Copyright 1999 Charles J. Hagenbuch <chuck@horde.org>
 
 You should have received a copy of the GNU Public
 License along with this package; if not, write to the
 Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA.
 
 */

/* display_toaddress: check the user's address and list of addresses that To: should be displayed for */
function display_toaddress ($user, $server, $from) {
   return is_int(strpos($from, get_barefrom($user, $server)));
}


$need_horde_db = 1;
$db_lib_loaded = 0;
require ('../lib/horde.lib');
require ('./lib/imp.lib');
require ('./lib/mimetypes.lib');
require ('./config/defaults.php3');

/* Html styles configuration */
require('./config/html.php3');
/* Mailbox configuration */
require('./config/mailbox.php3');
/* include mime and language configuration */
require('./config/mime.php3');

$this_client = new WebClient;

error_reporting($default->error_level);

/* Set or read session */
$session = create_session_object();
if (isset($actionID) && ($actionID == IMP_LOGIN) ) {
   $user     = $session->set_user($user);
   $pass     = $session->set_password($pass);
   $server   = $session->set_server($server);
   $port     = $session->set_port($port);
   $folders  = $session->set_folder($folders);
   $language = imp_get_lang($user, $server);
   if (isset($language) && $language) {
     $language = $session->set_lang($language);
   }
} else {
   $user     = $session->get_user();
   $pass     = $session->get_password();
   $server   = $session->get_server();
   $port     = $session->get_port();
   $folders  = $session->get_folder();
   $language = $session->get_lang();
}

if (!isset($language) || !$language) {
  $language = select_lang();
}
require ("./locale/$language/mailbox.lang");
require ('./locale/local/mailbox.lang');
caching_headers();

if (!isset($mailbox)) $mailbox = 'INBOX';
if (!isset($sortby)) $sortby = $default->sort_crit;
if (!isset($sort_reverse)) $sort_reverse = $default->sort_dir;

$current_mbox = str_replace("{$server}", '', $mailbox);
if (!empty($folders)) $current_mbox = str_replace($folders, '', $mailbox);
if (strlen($default->personal_folders) > 0) $current_mbox = str_replace($default->personal_folders, '', $current_mbox);

/* only proceed on a valid login */
$mbox = validate_login($server, $port, $mailbox, $user, $pass);

/* indicates invalid login or expired login so skip all the crap below */
if (isset($mbox) && (!$mbox)) exit;

if (!isset($plural)) $plural = false;

/* Run through the action handlers */

$status_string = '';
$status_color = 'green';

if (isset($actionID)) {
   if (isset($indices) && !is_array($indices)) { $indices = Array($indices); }
   
   switch ($actionID) {
    case NO_ACTION:
      $status_string = $lang->mailbox . $current_mbox;
      break;
      
    case DELETE_MESSAGES:
      if (isset($indices) && $indices && $mailbox) {
	 for ($i = 0; $i < count($indices); $i++) {
	    if (!(imap_delete($mbox, imap_msgno($mbox, $indices[$i])))) {
	       $status_string = $lang->error($current_mbox, 'deletion');
	       $status_color = 'red';
	    } else {
		if ($default->expunge_on_delete)
		    @imap_expunge($mbox);
	    }
	 }
	 if (!$status_string) {
	    if ($i != 1) $plural = true;
	    $status_string = $lang->success1($current_mbox, 'deleted', $i, $plural);
	 }
      }
      if (!$status_string) {
	 $status_string = $lang->error($current_mbox, 'deletion');
	 $status_color = 'red';
      }
      break;
      
    case UNDELETE_MESSAGES:
      if (isset($indices) && $mailbox && $indices) {
	 for ($i = 0; $i < count($indices); $i++) {
	    if (!(imap_undelete($mbox, imap_msgno($mbox, $indices[$i])))) {
	       $status_string = $lang->error($current_mbox, 'undeletion');
	       $status_color = 'red';
	    }
	 }
	 if (!$status_string) {
	    if ($i != 1) $plural = true;
	    $status_string = $lang->success1($current_mbox, 'undeleted', $i, $plural);
	 }
      }
      if (!$status_string) {
	 $status_string = $lang->error($current_mbox, 'undeletion');
	 $status_color = 'red';
      }
      break;
      
    case MOVE_MESSAGES:
      if (isset($indices) && isset($targetMbox) && $targetMbox && $indices) {
	 $msg_list = implode($indices, ',');
	 if (! (imap_mail_copy($mbox, $msg_list, $targetMbox, CP_UID|CP_MOVE)) ) {
	    $status_string = $lang->error($current_mbox, 'move');
	    $status_color = 'red';
	 } else {
	    if (count($indices) != 1) $plural = true;
	    $status_string = $lang->success2($current_mbox, $targetMbox, 'moved', count($indices), $plural);
	    if ($default->expunge_on_delete)
		@imap_expunge($mbox);
	 }
      }
      if (!$status_string) {
	 $status_string = $lang->error($current_mbox, 'move');
	 $status_color = 'red';
      }
      break;
      
    case COPY_MESSAGES:
      if (isset($targetMbox) && isset($indices) && $targetMbox && $indices) {
	 $msg_list = implode($indices, ',');
	 if (!(imap_mail_copy($mbox, $msg_list, $targetMbox, CP_UID)) ) {
            $status_string = $lang->error($current_mbox, 'copy');
	    $status_color = 'red';
	 } else {
	    if (count($indices) != 1) $plural = true;
	    $status_string = $lang->success2($current_mbox, $targetMbox, 'copied', count($indices), $plural);
	 }
      }
      if (!$status_string) {
	 $status_string = $lang->error($current_mbox, 'copy');
	 $status_color = 'red';
      }
      break;
      
    case EXPUNGE_MAILBOX:
      if ($mailbox) {
	 if (!(imap_expunge($mbox))) {
            $status_string = $lang->error($current_mbox, 'expunge');
	    $status_color = 'red';
	 } else {
	    $status_string = $lang->mailbox . ': ' . $current_mbox . ': ' . $lang->expunged;
	 }
      }
      if (!$status_string) {
	 $status_string = $lang->error($current_mbox, 'expunge');
	 $status_color = 'red';
      }
      break;
      
    case IMP_LOGIN:
      $status_string = $lang->mailbox . ': ' . $current_mbox;
      if (isset($new_lang)) {
         $session->set_lang($new_lang);
         $language = $new_lang;
      }
      break;
   }
   
} else $status_string = $lang->mailbox . ': ' . $current_mbox;


$check = imap_check($mbox);
$options = mailbox_list($mailbox, true);

/* mailbox list may open the stream OP_HALFREAD, this logs us into the mailbox again */
/* FIXME this is a hack and should be one using a session lock on the mailbox */
/* basically, we need to separate authentication and mailbox opening */
imap_close($mbox); // this might fix a few status things
$mbox = validate_login($server, $port, $mailbox, $user, $pass);

if ($check->Nmsgs != 0) $sorted = imap_sort($mbox, $sortby, $sort_reverse, SE_UID);
else $sorted = array();
$session->set_msgl($sorted);

$sidebar = false;
require ("$default->include_dir/doctype.inc");    /* doctype */

require ("$default->include_dir/generic-header.inc");
$nummsgs = $check->Nmsgs;  /* removed call to imap_num_msgs - cbj */

if ($nummsgs > $default->max_messages_page) {
   $page_count = ceil($nummsgs/$default->max_messages_page);
   
   /* Figure out which page to display */
   if (isset($page) && $page) {
      /* actually, do need to do something here: make sure we're not past the end */
      if ($page > $page_count) $page = $page_count;
   } elseif (isset($start) && $start) {
      /* no idea where/if this is used - cbj */
      /* messages set it when returning to a mailbox - cjh */
      $page = ceil($start/$default->max_messages_page);
   } else {
      $page = 1;
   }
   $begin = ($page - 1) * $default->max_messages_page + 1;
   
   $end = $begin + $default->max_messages_page - 1;
   if ($end > $nummsgs) { $end = $nummsgs; }
   
   if ($page == 1) {
      if (!$this_client->lynx) {
	 $pages_first = "<img src=\"$default->graphics_url/first-grey.gif\" border=\"0\" alt=\"\">";
	 $pages_prev  = "<img src=\"$default->graphics_url/prev-grey.gif\" border=\"0\" alt=\"\">";
      } else {
	 $pages_first = '';
	 $pages_prev  = '';
      }
   } else {
      $prev             = $page - 1;
      $target_first_url = "$default->root_url/mailbox.php3?mailbox=" . urlencode($mailbox) . "&page=1&sortby=$sortby";
      $target_prev_url  = "$default->root_url/mailbox.php3?mailbox=" . urlencode($mailbox) . "&page=$prev&sortby=$sortby";
      if (!$this_client->lynx) {
	 $pages_first = "<a href=\"$target_first_url\"><img src=\"$default->graphics_url/first.gif\" border=\"0\" alt=\"first page\"></a>";
	 $pages_prev  = "<a href=\"$target_prev_url\"><img src=\"$default->graphics_url/prev.gif\" border=\"0\" alt=\"prev page\"></a>";
      } else {
	 $pages_first = "<a href=\"$target_first_url\">&lt;&lt; [ First Page ]";
	 $pages_prev  = "<a href=\"$target_prev_url\">&lt; [ Previous Page ]</a>";
      }
   }
   
   if ($page == $page_count) {
      if (!$this_client->lynx) {
	 $pages_last = "<img src=\"$default->graphics_url/last-grey.gif\" border=\"0\" alt=\"\">";
	 $pages_next = "<img src=\"$default->graphics_url/next-grey.gif\" border=\"0\" alt=\"\">"; 
      } else {
	 $pages_last = '';
	 $pages_next = '';
      }
   } else {
      $next = $page + 1;
      $target_last_url = "$default->root_url/mailbox.php3?mailbox=" . urlencode($mailbox) . "&page=$page_count&sortby=$sortby";
      $target_next_url = "$default->root_url/mailbox.php3?mailbox=" . urlencode($mailbox) . "&page=$next&sortby=$sortby";
      
      if (!$this_client->lynx) {
	 $pages_last = "<a href=\"$target_last_url\"><img src=\"$default->graphics_url/last.gif\" border=\"0\" alt=\"last page\"></a>";
	 $pages_next = "<a href=\"$target_next_url\"><img src=\"$default->graphics_url/next.gif\" border=\"0\" alt=\"next page\"></a>";
      } else {
	 $pages_last = "<a href=\"$target_last_url\">[ Last Page ] &gt;&gt;</a>";
	 $pages_next = "<a href=\"$target_next_url\">[ Next Page ] &gt;</a>";
      }
      
   }
   
} else {
   $begin = 1;
   $end   = $nummsgs;
   $page  = 1;
}

$title = "Index of $current_mbox";

if ($this_client->lynx) {
   include_lynx_menu(IMPMENU_MAIN);
   include("$default->include_dir/mailbox/lynx_header.inc");
   include("$default->include_dir/mailbox/lynx_navbar_top.inc");
   include("$default->include_dir/mailbox/lynx_actions.inc");
   include("$default->include_dir/mailbox/lynx_message_headers.inc");
} else {
   include("$default->include_dir/mailbox/header.inc");
   include("$default->include_dir/mailbox/navbar_top.inc");
   include("$default->include_dir/mailbox/actions.inc");
   include("$default->include_dir/mailbox/message_headers.inc");
}

// hack to read headers twice if there is new mail, to try and get the correct status
$mailinfo = @imap_mailboxmsginfo($mbox);
if ($mailinfo) {
   $num_recent = $mailinfo->Recent;
   if (isset($num_recent) && $num_recent > 0) {
      imap_close($mbox);
      $mbox = imap_open('{' . "$server:$port" . '}' . $mailbox, $user, $pass, OP_READONLY);
   }
}

// now step through each message
for ($i=$begin; $i <= $end; $i++) {
   
   $msgnum = $sorted[$i-1];
   $h = @imap_header($mbox, imap_msgno($mbox, $msgnum));
   
   /* initialize the header fields */
   $dat = '';
   $frm = '';
   $sub = '';
   $msg_size = '?';
   
   /* now pull the IMAP header values into them, decoding and chopping them at the same time */
   /* formats the header date string nicely */
   /* ignores the timezone info for now     */
   /* thanks to David R Bosso <dbosso@ltsc.ucsb.edu> */
   if (isset($h->date)) {
     $date = $h->date;
     if (ord($date) > 64)    // nuke leading day if it exists
       $date = substr($date, 5);
     while (strstr('  ', $date)) {
       $date = str_replace('  ', ' ', $date);
     }
     $dateparts = explode(' ', $date);
     if (strlen($dateparts[0]) == 1)    // pad days with 0
       $dateparts[0] = '0' . $dateparts[0];
     // cjh: compensate for messages with the year in the wrong place (mostly IMP's fault; fixed now)
     if (strlen($dateparts[2]) != 2 && strlen($dateparts[2]) != 4) {
       $tmp = $dateparts[2];
       $dateparts[2] = $dateparts[3];
       $dateparts[3] = $tmp;
     }
     if (strlen($dateparts[2]) == 2)    // get 4 digit year
       if ($dateparts[2] > 70)
	 $dateparts[2] = '19' . $dateparts[2];
       else $dateparts[2] = '20'.$dateparts[2];
     $timeparts = explode(':', $dateparts[3]);  // separate time parts
     if (strlen($timeparts[0]) == 1)    // pad hours with 0
       $timeparts[0] = '0' . $timeparts[0];
     if (!isset($timeparts[1]))
       $timeparts[1] = '00';
     $dateparts[3] = $timeparts[0] . ':' . $timeparts[1];  // time - HH:MM
     switch ($dateparts[1]) {      // get a month number
     case 'Jan':
       $monthnum='01';
       break;
     case 'Feb':
       $monthnum='02';
       break;
     case 'Mar':
       $monthnum='03';
       break;
     case 'Apr':
       $monthnum='04';
       break;
     case 'May':
       $monthnum='05';
       break;
     case 'Jun':
       $monthnum='06';
       break;
     case 'Jul':
       $monthnum='07';
       break;
     case 'Aug':
       $monthnum='08';
       break;
     case 'Sep':
       $monthnum='09';
       break;
     case 'Oct':
       $monthnum='10';
       break;
     case 'Nov':
       $monthnum='11';
       break;
     default:
       $monthnum='12';
     }
     
     // set $date for sorting
     $date = $dateparts[2] . $monthnum . $dateparts[0] . $timeparts[0] . $timeparts[1];
     if ($dateparts[2]!=date('Y')) {  // if it's not this year
       // MM/DD/YY
       $dat = $monthnum . '/' . $dateparts[0] . '/' . substr($dateparts[2], 2);
     } else if ((date('M') != $dateparts[1]) || (date('d') != $dateparts[0])) {
       // not today
       $dat = $dateparts[0] . ' ' . $dateparts[1]; // use DD MMM
     } else { // else it's today
       $dat = $dateparts[3];  // use the time HH:MM
     }
   }  
   
   if (isset($h->from[0])) {
     $from = $h->from[0];
     if (isset($from->personal)) {
       $frm = trim(decode_mime_string($from->personal));
     } else if (isset($from->mailbox) && isset($from->host)) {
       $frm = $from->mailbox . '@' . $from->host;
     } else if (isset($h->fromaddress))
       $frm = trim(decode_mime_string($h->fromaddress));
   } else if (isset($h->fromaddress)) $frm = trim(decode_mime_string($h->fromaddress));
   
   if (isset($h->fromaddress) && display_toaddress($user, $server, $h->fromaddress)) {
     if (isset($h->toaddress))
       $to = trim(decode_mime_string($h->toaddress));
     else
       $to = $lang->undisclosed_recipients;
     $frm = $lang->to_label . ': ' . $to;
   }
   
   if (isset($h->subject)) $sub = trim(decode_mime_string($h->subject));
   if (isset($h->Size)) $msg_size = ($h->Size > 1024) ? sprintf("%.0f kb", $h->Size / 1024) : $h->Size;
   if (strlen($frm) > $default->max_frm_chars) $frm = substr($frm, 0, $default->max_frm_chars) . '...';
   if (strlen($sub) > $default->max_sub_chars) $sub = substr($sub, 0, $default->max_sub_chars) . '...';
   
   /* make sure that they get put into an HTML readable format */
   $frm = htmlspecialchars($frm);
   $sub = htmlspecialchars($sub);
   
   /* set them to useful things for IMP if they ended up being empty */
   if ($dat == '') { $dat = '&nbsp;'; }
   if ($frm == '') { $frm = '&nbsp;'; }
   if ($sub == '') { $sub = $lang->no_subject; } // make sure to have something here for the link text
   
   $icons = array();
   
   if ($default->fetch_mime_info) {
      $flag = array();
      
      $structure = @imap_fetchstructure($mbox,imap_msgno($mbox, $msgnum));
      
      $num_parts = @count($structure->parts);
      $bodypart = 1;
      $j = 0;
      if (is_array($structure->parts) && $num_parts > 0) {
	if ($num_parts > 9) $num_parts = 9;
	
	while ($j <= $num_parts) {
	  if (is_object($structure->parts[$j])) {
	    $mime = new mime_part_data;
	    $mime->set_imap_data($mailbox, $msgnum, $j);
	    $mime->parse( $structure->parts[$j] );
	    if ($mime->type_text != 'text' || $mime->subtype != 'plain' || $j != 0 ) {
	      if (isset($icons[$mime->text]) )
		$icons[$mime->text]["count"]++;
	      else {
		$icons[$mime->text]['display'] = $mime->img_tag();
		$icons[$mime->text]['count'] = 1;
	      }
	    }
	  }
	  $j++;
	}
	for (reset($icons); $key=key($icons); next($icons)) {
	  $tmp = $icons[$key]['display'];
	  if ($icons[$key]["count"] > 1 ) 
	    $tmp .= '<subscript><b>' . $icons[$key]['count'] . '</b></subscript>';
	  $tmp .= "\n";
	  $flag[] = $tmp;
	}
	
      } else {
	
	/* Only one part in message, check for an attachment */
	$mime = new mime_part_data;
	$mime->set_imap_data($mailbox, $msgnum, $j);
	$mime->parse($structure);
	if ($mime->type_text != 'text' || $mime->subtype != 'plain')
	  $flag[] = $mime->img_tag();
      }
      
   }
   
   include( "$default->include_dir/mailbox/message_summaries.inc" );
}

require("$default->include_dir/mailbox/message_footers.inc");
require("$default->include_dir/mailbox/actions.inc");
require("$default->include_dir/mailbox/navbar_bottom.inc");
require("$default->include_dir/mailbox/footer.inc");
require('./lib/js/open_compose_win.js');
require("$default->include_dir/mailbox/javascript.inc");
require('./lib/js/open_contacts.js');

if ((isset($actionID) && ($actionID == IMP_LOGIN)) ||
    (isset($horde_frames))) {
    include( "$default->include_dir/mailbox/refresh_toolbar-javascript.inc" );
}

if ( ! $this_client->lynx ) {
status($status_string, $status_color);
}

?>
